/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2021 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.14.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : OS_UltraComparison.c
Purpose : This demo application shows how embOS Ultra reduces the amount
          of necessary hardware timer interrupts which is much more power
          efficient.

          embOS uses a fixed system tick hardware timer interrupt occurring
          every millisecond. embOS Ultras instead programs the hardware timer
          as required by the application only.
*/

#include "RTOS.h"
#include "BSP.h"
#include <stdio.h>

#define HPTASK_DELAY               50  // Task delay in milliseconds
#define LPTASK_DELAY              201  // Task delay in milliseconds
#define MILLISECONDS_PER_SECOND  1000

static OS_STACKPTR int StackHP[128], StackLP[128];  // Task stacks
static OS_TASK         TCBHP, TCBLP;                // Task control blocks
static OS_U32          i;

extern volatile OS_U32 TickCnt;

static void HPTask(void) {
  i = 0u;
  while (1) {
    BSP_ToggleLED(0);
    OS_TASK_Delay(HPTASK_DELAY);
    i++;
    //
    // Print every second the amount of system tick interrupts
    //
    if (i == MILLISECONDS_PER_SECOND / HPTASK_DELAY) {
      printf("System tick interrupts per second: %u \n", TickCnt);
      TickCnt = 0u;
      i       = 0u;
    }
  }
}

static void LPTask(void) {
  while (1) {
    BSP_ToggleLED(1);
    OS_TASK_Delay(LPTASK_DELAY);
  }
}

/*********************************************************************
*
*       main()
*/
int main(void) {
  OS_Init();    // Initialize embOS
  OS_InitHW();  // Initialize required hardware
  BSP_Init();   // Initialize LED ports
  OS_TASK_CREATE(&TCBHP, "HP Task", 100, HPTask, StackHP);
  OS_TASK_CREATE(&TCBLP, "LP Task",  50, LPTask, StackLP);
  OS_Start();   // Start embOS
  return 0;
}

/*************************** End of file ****************************/
